#!/usr/bin/env node
import { program } from 'commander';
import { Effect } from 'effect';
import { io } from '../io/index.js';
import { showHelpOnError } from '../lib/show-help-on-error.js';
import { option } from '../option.js';
import { listMismatches } from './list-mismatches.js';
program.description(`
  List dependencies which are required by multiple packages, where the version
  is not the same across every package.`.replace(/^\n/, ''));
program.on('--help', () => { });
showHelpOnError(program);
program
    .option(...option.source)
    .option(...option.filter)
    .option(...option.config)
    .option(...option.specs)
    .option(...option.types)
    .parse(process.argv);
Effect.runPromise(listMismatches({
    io,
    cli: {
        configPath: program.opts().config,
        filter: program.opts().filter,
        source: program.opts().source,
        specs: program.opts().specs,
        types: program.opts().types,
    },
}));
